@@ -11,12 +11,14 @@ from os.path import join, abspath, dirname, exists
 import sys
 
 from math import inf
-from numpy import arange, array
+from numpy import array
 
 class Config(object):
 
-    # Output level. 'QUIET': nothing, 'NORMAL': warn, err,
-    # 'DEBUG': info, warn, err
+    # Output level
+    #   'QUIET': nothing
+    #   'NORMAL': warn, err,
+    #   'DEBUG': info, warn, err
     _out_level = 'NORMAL'
 
     # Timestamp for log output
@@ -33,13 +35,19 @@ class Config(object):
     if 'IPython' in sys.modules: _ipython_avail = True
     else: _ipython_avail = False
 
-    # Default location for timezones
-    _location = 'Europe/Madrid'
+    # Default timezone
+    _timezone = 'Europe/Madrid'
     _epsg = 4326
 
     # Returns when iterables cannot be fully processed
     _strict = False
 
+    # Ignore additional channels in csv loads
+    _strict_load = True
+
+    # Timeout for http requests
+    _timeout = 3
+
     ### ---------------------------------------
     ### ----------------CRONTAB----------------
     ### ---------------------------------------
@@ -53,6 +61,7 @@ class Config(object):
     # Tasks
     _postprocessing_interval_hours = 1
     _device_processor = 'dprocess'
+    _max_forward_retries = 2
 
     ### ---------------------------------------
     ### -----------------DATA------------------
@@ -93,6 +102,22 @@ class Config(object):
     # Plot out level (priority of the plot to show - 'DEBUG' or 'NORMAL')
     _plot_out_level = 'DEBUG'
 
+    # Alphasense sensor codes
+    _as_sensor_codes =  {
+        '132':  'ASA4_CO',
+        '133':  'ASA4_H2S',
+        '130':  'ASA4_NO',
+        '212':  'ASA4_NO2',
+        '214':  'ASA4_OX',
+        '134':  'ASA4_SO2',
+        '162':  'ASB4_CO',
+        '133':  'ASB4_H2S',#
+        '130':  'ASB4_NO', #
+        '202':  'ASB4_NO2',
+        '204':  'ASB4_OX',
+        '164':  'ASB4_SO2'
+    }
+
     ### ---------------------------------------
     ### ----------------ZENODO-----------------
     ### ---------------------------------------
@@ -105,7 +130,8 @@ class Config(object):
     ### -------------SMART CITIZEN-------------
     ### ---------------------------------------
     # # Urls
-    sensor_names_url_21='https://raw.githubusercontent.com/fablabbcn/smartcitizen-kit-21/master/lib/Sensors/Sensors.h'
+    sensor_names_url_21 = 'https://raw.githubusercontent.com/fablabbcn/smartcitizen-kit-21/master/lib/Sensors/Sensors.h'
+    sensors_api_names_url = 'https://api.smartcitizen.me/v0/sensors/?per_page=500'
     # sensor_names_url_20='https://raw.githubusercontent.com/fablabbcn/smartcitizen-kit-20/master/lib/Sensors/Sensors.h'
     _base_postprocessing_url = 'https://raw.githubusercontent.com/fablabbcn/smartcitizen-data/master/'
     _default_file_type = 'json'
@@ -128,7 +154,8 @@ class Config(object):
         f'{_base_postprocessing_url}blueprints/sck_21.{_default_file_type}',
         f'{_base_postprocessing_url}blueprints/sck_21_gps.{_default_file_type}',
         f'{_base_postprocessing_url}blueprints/sck_21_nilu.{_default_file_type}',
-        f'{_base_postprocessing_url}blueprints/sck_21_co2.{_default_file_type}'
+        f'{_base_postprocessing_url}blueprints/sck_21_co2.{_default_file_type}',
+        f'{_base_postprocessing_url}blueprints/sc_21_water.{_default_file_type}'
     ]
 
     connectors_urls = [
@@ -149,12 +176,6 @@ class Config(object):
         ['ms','ms']
     )
 
-    # AlphaDelta PCB factor (converstion from mV to nA)
-    _alphadelta_pcb = 6.36
-    # Deltas for baseline deltas algorithm
-    _baseline_deltas = arange(30, 45, 5)
-    # Lambdas for baseline ALS algorithm
-    _baseline_als_lambdas = [1e5]
 
     ### ---------------------------------------
     ### -------------METRICS DATA--------------
@@ -179,95 +200,6 @@ class Config(object):
         'H2S':  34
     }
 
-    # Background concentrations
-    _background_conc = {
-        'CO':   0,
-        'NO2':  8,
-        'O3':   40
-    }
-
-    # Alphasense data
-    _as_sensor_codes =  {
-        '132':  'ASA4_CO',
-        '133':  'ASA4_H2S',
-        '130':  'ASA4_NO',
-        '212':  'ASA4_NO2',
-        '214':  'ASA4_OX',
-        '134':  'ASA4_SO2',
-        '162':  'ASB4_CO',
-        '133':  'ASB4_H2S',#
-        '130':  'ASB4_NO', #
-        '202':  'ASB4_NO2',
-        '204':  'ASB4_OX',
-        '164':  'ASB4_SO2'
-    }
-
-    # From Tables 2 and 3 of AAN 803-04
-    _as_t_comp = [-30, -20, -10, 0, 10, 20, 30, 40, 50]
-
-    _as_sensor_algs = {
-        'ASA4_CO':
-                    {
-                        1: ['n_t',      [1.0, 1.0, 1.0, 1.0, -0.2, -0.9, -1.5, -1.5, -1.5]],
-                        4: ['kpp_t',    [13, 12, 16, 11, 4, 0, -15, -18, -36]],
-                    },
-        'ASA4_H2S':
-                    {
-                        2: ['k_t',      [-1.5, -1.5, -1.5, -0.5, 0.5, 1.0, 0.8, 0.5, 0.3]],
-                        1: ['n_t',      [3.0, 3.0, 3.0, 1.0, -1.0, -2.0, -1.5, -1.0, -0.5]]
-                    },
-        'ASA4_NO':
-                    {
-                        3: ['kp_t',     [0.7, 0.7, 0.7, 0.7, 0.8, 1.0, 1.2, 1.4, 1.6]],
-                        4: ['kpp_t',    [-25, -25, -25, -25, -16, 0, 56, 200, 615]]
-                    },
-        'ASA4_NO2':
-                    {
-                        1: ['n_t',      [0.8, 0.8, 1.0, 1.2, 1.6, 1.8, 1.9, 2.5, 3.6]],
-                        3: ['kp_t',     [0.2, 0.2, 0.2, 0.2, 0.7, 1.0, 1.3, 2.1, 3.5]]
-                    },
-        'ASA4_OX':
-                    {
-                        3: ['kp_t',     [0.1, 0.1, 0.2, 0.3, 0.7, 1.0, 1.7, 3.0, 4.0]],
-                        1: ['n_t',      [1.0, 1.2, 1.2, 1.6, 1.7, 2.0, 2.1, 3.4, 4.6]]
-                    },
-        'ASA4_SO2':
-                    {
-                        4: ['kpp_t',    [0, 0, 0, 0, 0, 0, 5, 25, 45]],
-                        1: ['n_t',      [1.3, 1.3, 1.3, 1.2, 0.9, 0.4, 0.4, 0.4, 0.4]]
-                    },
-        'ASB4_CO':
-                    {
-                        1: ['n_t',      [0.7, 0.7, 0.7, 0.7, 1.0, 3.0, 3.5, 4.0, 4.5]],
-                        2: ['k_t',      [0.2, 0.2, 0.2, 0.2, 0.3, 1.0, 1.2, 1.3, 1.5]]
-                    },
-        'ASB4_H2S':
-                    {
-                        1: ['n_t',      [-0.6, -0.6, 0.1, 0.8, -0.7, -2.5, -2.5, -2.2, -1.8]],
-                        2: ['k_t',      [0.2, 0.2, 0.0, -0.3, 0.3, 1.0, 1.0, 0.9, 0.7]]
-                    },
-        'ASB4_NO':
-                    {
-                        2: ['k_t',      [1.8, 1.8, 1.4, 1.1, 1.1, 1.0, 0.9, 0.9, 0.8]],
-                        3: ['kp_t',     [0.8, 0.8, 0.8, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3]]
-                    },
-        'ASB4_NO2':
-                    {
-                        1: ['n_t',      [1.3, 1.3, 1.3, 1.3, 1.0, 0.6, 0.4, 0.2, -1.5]],
-                        3: ['kp_t',     [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.4, -0.1, -4.0]]
-                    },
-        'ASB4_OX':
-                    {
-                        1: ['n_t',      [0.9, 0.9, 1.0, 1.3, 1.5, 1.7, 2.0, 2.5, 3.7]],
-                        3: ['kp_t',     [0.5, 0.5, 0.5, 0.6, 0.6, 1.0, 2.8, 5.0, 5.3]]
-                    },
-        'ASB4_SO2':
-                    {
-                        4: ['kpp_t',    [-4, -4, -4, -4, -4, 0, 20, 140, 450]],
-                        1: ['n_t',      [1.6, 1.6, 1.6, 1.6, 1.6, 1.6, 1.9, 3.0, 5.8]]
-                    }
-        }
-
     # This look-up table is comprised of channels you want always want to have with the same units and that might come from different sources
     # i.e. pollutant data in various units (ppm or ug/m3) from different analysers
     # The table should be used as follows:
@@ -358,11 +290,17 @@ class Config(object):
             'xrange': None,
             'title': None,
             'suptitle_factor': 0.92,
+            'suptitle_x': 0.5,
+            'suptitle_y': 0.98,
+            'title_loc': "center",
+            'title_fontsize': 14,
+            'hspace':0.4,
+            'wspace':0.4,
             'grid': True,
             'fontsize': 10,
-            'title_fontsize': 14,
             'alpha_highlight': 0.8,
             'alpha_other': 0.1,
+            'alpha_bands': 0.2,
             'palette': None,
             'decorators': None,
             'legend': True,
@@ -525,7 +463,9 @@ class Config(object):
             'fontsize': 8.,
             'orientation': 'left',
             'title_fontsize': 14,
-            'suptitle_factor': 0.92
+            'suptitle_factor': 0.92,
+            'style': _plot_style,
+            'palette': None
         }
     }
 
@@ -594,6 +534,12 @@ class Config(object):
         'hardware_loaded_from_url'
     ]
 
+    _csv_defaults = {
+        'index_name': 'TIME',
+        'sep': ',',
+        'skiprows': None
+    }
+
     def __init__(self):
         self._env_file = False
         self.paths = get_paths()
@@ -648,6 +594,7 @@ class Config(object):
         namespath = join(self.paths['interim'], 'names.json')
         if self.data['reload_metadata'] or not exists(namespath):
             sc_sensor_names = load_firmware_names(self.sensor_names_url_21)
+            # sc_sensor_names = load_api_names(self.sensors_api_names_url)
             namesreload = True
         else:
             with open(namespath, 'r') as file: sc_sensor_names = json.load(file)
