@@ -5,12 +5,14 @@ from scdata.io import read_csv_file, export_csv_file
 from scdata.utils import LazyCallable, url_checker, get_json_from_url
 from scdata._config import config
 from scdata.device.process import *
+from scdata.io.device_api import *
 
 from os.path import join, basename
 from urllib.parse import urlparse
 from pandas import DataFrame, to_timedelta
 from traceback import print_exc
 from numpy import nan
+from collections.abc import Iterable
 
 class Device(object):
     ''' Main implementation of the device class '''
@@ -118,7 +120,9 @@ class Device(object):
         for ditem in self.description.keys():
             if ditem not in vars(self): std_out (f'Ignoring {ditem} from input'); continue
             if type(self.__getattribute__(ditem)) == dict:
-                self.__setattr__(ditem, dict_fmerge(self.__getattribute__(ditem), self.description[ditem]))
+                self.__setattr__(ditem,
+                    dict_fmerge(self.__getattribute__(ditem),
+                    self.description[ditem]))
             else: self.__setattr__(ditem, self.description[ditem])
 
     def check_overrides(self, options = {}):
@@ -145,6 +149,11 @@ class Device(object):
         else:
             self.options['frequency'] = '1Min'
 
+        if 'resample' in options.keys():
+            self.options['resample'] = options['resample']
+        else:
+            self.options['resample'] = True
+
     def load_postprocessing(self):
 
         if self.source != 'api': return None
@@ -245,7 +254,28 @@ class Device(object):
         if self.hardware_description is not None: return True
         else: return False
 
-    def load(self, options = None, path = None, convert_units = True, only_unprocessed = False, max_amount = None):
+    def merge_sensor_metrics(self, ignore_empty = True):
+        std_out('Merging sensor and metrics channels')
+        all_channels = dict_fmerge(self.sensors, self.metrics)
+
+        if ignore_empty:
+            to_ignore = []
+            for channel in all_channels.keys():
+                if channel not in self.readings: to_ignore.append(channel)
+                elif self.readings[channel].dropna().empty:
+                    std_out (f'{channel} is empty')
+                    to_ignore.append(channel)
+
+            [all_channels.pop(x) for x in to_ignore]
+
+        return all_channels
+
+    def make_raw(self):
+
+        self.processed_data_file = self.raw_data_file
+        return True
+
+    def load(self, options = None, path = '', convert_units = True, only_unprocessed = False, max_amount = None, follow_defaults = False):
         '''
         Loads the device with some options
 
@@ -262,7 +292,7 @@ class Device(object):
             options['frequency'] = frequency to load data at in pandas format
                 Default to device frequency (from blueprint or test) or '1Min'
         path: String
-            Default: None
+            Default: ''
             Path were the csv file is, if any. Normally not needed to be provided, only for internal usage
         convert_units: bool
             Default: True
@@ -273,6 +303,9 @@ class Device(object):
         max_amount: int
             Default: None
             Trim dataframe to this amount for processing and forwarding purposes
+        follow_defaults: bool
+            Default: False
+            Use defaults from config._csv_defaults for loading
         Returns
         ----------
             True if loaded correctly
@@ -282,20 +315,50 @@ class Device(object):
         if options is not None: self.check_overrides(options)
         else: self.check_overrides()
 
+        std_out(f'Using options for device: {options}')
+
         try:
             if self.source == 'csv':
-                self.readings = self.readings.combine_first(read_csv_file(join(path, self.processed_data_file), self.location,
-                                                            self.options['frequency'], self.options['clean_na'],
-                                                            self.sources[self.source]['index']))
+                if follow_defaults:
+                    index_name = config._csv_defaults['index_name']
+                    sep = config._csv_defaults['sep']
+                    skiprows = config._csv_defaults['skiprows']
+                else:
+                    index_name = self.sources[self.source]['index']
+                    sep = self.sources[self.source]['sep']
+                    skiprows = self.sources[self.source]['header_skip']
+
+                # here we don't use tzaware because we only load preprocessed data
+                self.readings = self.readings.combine_first(
+                                                        read_csv_file(
+                                                            file_path = join(path, self.processed_data_file),
+                                                            timezone = self.timezone,
+                                                            frequency = self.options['frequency'],
+                                                            clean_na = self.options['clean_na'],
+                                                            index_name = index_name,
+                                                            sep = sep,
+                                                            skiprows = skiprows,
+                                                            resample = self.options['resample'])
+                                                        )
                 if self.readings is not None:
                     self.__convert_names__()
 
             elif 'api' in self.source:
 
                 # Get device location
-                self.location = self.api_device.get_device_timezone()
+                # Location data should be standard for each new device
+                self.api_device.get_device_lat_long()
+                self.api_device.get_device_alt()
 
-                if path is None:
+                self.location = {
+                    'longitude': self.api_device.long,
+                    'latitude': self.api_device.lat,
+                    'altitude': self.api_device.alt
+                }
+
+                self.timezone = self.api_device.get_device_timezone()
+
+                if path == '':
                     # Not chached case
                     if only_unprocessed:
 
@@ -305,8 +368,11 @@ class Device(object):
                             # Override min loading date
                             self.options['min_date'] = hw_latest_postprocess
 
-                    df = self.api_device.get_device_data(self.options['min_date'], self.options['max_date'],
-                                                         self.options['frequency'], self.options['clean_na'])
+                    df = self.api_device.get_device_data(self.options['min_date'],
+                                                         self.options['max_date'],
+                                                         self.options['frequency'],
+                                                         self.options['clean_na'],
+                                                         resample = self.options['resample'])
 
                     # API Device is not aware of other csv index data, so make it here
                     if 'csv' in self.sources and df is not None:
@@ -318,32 +384,33 @@ class Device(object):
                 else:
                     # Cached case
                     self.readings = self.readings.combine_first(read_csv_file(join(path, str(self.id) + '.csv'),
-                                                                self.location, self.options['frequency'],
-                                                                self.options['clean_na'], self.sources['csv']['index']))
+                                                                self.timezone, self.options['frequency'],
+                                                                self.options['clean_na'], self.sources['csv']['index'],
+                                                                resample = self.options['resample']))
 
         except FileNotFoundError:
+            # print_exc()
             # Handle error
             if 'api' in self.source: std_out(f'No cached data file found for device {self.id} in {path}. Moving on', 'WARNING')
             elif 'csv' in self.source: std_out(f'File not found for device {self.id} in {path}', 'ERROR')
 
             self.loaded = False
         except:
-            print_exc()
+            # print_exc()
             self.loaded = False
         else:
             if self.readings is not None:
                 self.__check_sensors__()
-                if max_amount is not None:
-                    std_out(f'Trimming dataframe to {max_amount} rows')
-                    self.readings=self.readings.dropna(axis = 0, how='all').head(max_amount)
                 if not self.readings.empty:
+                    if max_amount is not None:
+                        std_out(f'Trimming dataframe to {max_amount} rows')
+                        self.readings=self.readings.dropna(axis = 0, how='all').head(max_amount)
                     # Only add metrics if there is something that can be potentially processed
                     self.__fill_metrics__()
                     self.loaded = True
                     if convert_units: self.__convert_units__()
                 else:
                     std_out('Empty dataframe in readings', 'WARNING')
-
         finally:
             self.processed = False
             return self.loaded
@@ -369,19 +436,18 @@ class Device(object):
                     return None
 
                 for slot in version["ids"]:
-
                     # Alphasense type - AAN 803-04
                     if slot.startswith('AS'):
-
                         sensor_id = version["ids"][slot]
                         as_type = config._as_sensor_codes[sensor_id[0:3]]
-                        pollutant = as_type[as_type.index('_')+1:]
-                        if pollutant == 'OX': pollutant = 'O3'
+                        channel = as_type[as_type.index('_')+1:]
+                        pollutant = channel
+                        if channel == 'OX':
+                            pollutant = 'O3'
 
                         # Get working and auxiliary electrode names
                         wen = f"ADC_{slot.strip('AS_')[:slot.index('_')]}_{slot.strip('AS_')[slot.index('_')+1]}"
                         aen = f"ADC_{slot.strip('AS_')[:slot.index('_')]}_{slot.strip('AS_')[slot.index('_')+2]}"
-
                         if pollutant not in self.metrics:
                             # Create Metric
                             std_out(f'Metric {pollutant} not in blueprint, ignoring.', 'WARNING')
@@ -390,14 +456,42 @@ class Device(object):
                             std_out(f'{pollutant} found in blueprint metrics, filling up with hardware info')
                             self.metrics[pollutant]['kwargs']['we'] = wen
                             self.metrics[pollutant]['kwargs']['ae'] = aen
-                            self.metrics[pollutant]['kwargs']['location'] = self.location
+                            self.metrics[pollutant]['kwargs']['timezone'] = self.timezone
                             self.metrics[pollutant]['kwargs']['alphasense_id'] = str(sensor_id)
                             self.metrics[pollutant]['kwargs']['from_date'] = from_date
                             self.metrics[pollutant]['kwargs']['to_date'] = to_date
+                            # Add channel name for traceability
+                            self.metrics[f'{channel}_WE']['kwargs']['channel'] = wen
+                            self.metrics[f'{channel}_AE']['kwargs']['channel'] = aen
+
+                    # Alphasense type - AAN 803-04
+                    if slot.startswith('PT'):
+
+                        sensor_id = version["ids"][slot]
+
+                        # Get working and auxiliary electrode names
+                        pt1000plus = f"ADC_{slot.strip('PT_')[:slot.index('_')]}_{slot.strip('PT_')[slot.index('_')+1]}"
+                        pt1000minus = f"ADC_{slot.strip('PT_')[:slot.index('_')]}_{slot.strip('PT_')[slot.index('_')+2]}"
+
+                        metric = 'ASPT1000'
+                        if 'ASPT1000' not in self.metrics:
+                            # Create Metric
+                            std_out(f'Metric {metric} not in blueprint, ignoring.', 'WARNING')
+                        else:
+                            # Simply fill it up
+                            std_out(f'{metric} found in blueprint metrics, filling up with hardware info')
+                            self.metrics[metric]['kwargs']['pt1000plus'] = pt1000plus
+                            self.metrics[metric]['kwargs']['pt1000minus'] = pt1000minus
+                            self.metrics[metric]['kwargs']['afe_id'] = str(sensor_id)
+                            self.metrics[metric]['kwargs']['timezone'] = self.timezone
+                            self.metrics[metric]['kwargs']['from_date'] = from_date
+                            self.metrics[metric]['kwargs']['to_date'] = to_date
+                            # Add channel name for traceability
+                            self.metrics[f'PT1000_POS']['kwargs']['channel'] = pt1000plus
 
                     # Other metric types will go here
-            else:
-                std_out('No hardware versions found, ignoring additional metrics', 'WARNING')
+        else:
+            std_out('No hardware versions found, ignoring additional metrics', 'WARNING')
 
     def __check_sensors__(self):
         remove_sensors = list()
@@ -405,15 +499,25 @@ class Device(object):
         for sensor in self.sensors:
             if sensor not in self.readings.columns: remove_sensors.append(sensor)
 
-        if remove_sensors != []: std_out(f'Removing sensors from device: {remove_sensors}', 'WARNING')
-        for sensor_to_remove in remove_sensors: self.sensors.pop(sensor_to_remove, None)
+        if remove_sensors != []:
+            std_out(f'Removing sensors from device: {remove_sensors}', 'WARNING')
+        for sensor_to_remove in remove_sensors:
+            self.sensors.pop(sensor_to_remove, None)
+
+        if config._strict_load:
+            remove_columns = list()
+            for column in self.readings.columns:
+                if column not in self.sensors: remove_columns.append(column)
+            self.readings.drop(remove_columns, axis=1, inplace=True)
 
         std_out(f'Device sensors after removal: {list(self.sensors.keys())}')
 
     def __convert_names__(self):
         rename = dict()
         for sensor in self.sensors:
-            if 'id' in self.sensors[sensor] and sensor in self.readings.columns: rename[self.sensors[sensor]['id']] = sensor
+            if 'id' in self.sensors[sensor]:
+                if self.sensors[sensor]['id'] in self.readings.columns:
+                    rename[self.sensors[sensor]['id']] = sensor
         self.readings.rename(columns=rename, inplace=True)
 
     def __convert_units__(self):
@@ -428,15 +532,15 @@ class Device(object):
             factor = get_units_convf(sensor, from_units = self.sensors[sensor]['units'])
 
             if factor != 1:
-                self.readings.rename(columns={sensor: sensor + '_RAW'}, inplace=True)
-                self.readings.loc[:, sensor] = self.readings.loc[:, sensor + '_RAW']*factor
+                self.readings.rename(columns={sensor: sensor + '_in_' + self.sensors[sensor]['units']}, inplace=True)
+                self.readings.loc[:, sensor] = self.readings.loc[:, sensor + '_in_' + self.sensors[sensor]['units']]*factor
         std_out('Units check done', 'SUCCESS')
 
     def process(self, only_new = False, lmetrics = None):
         '''
         Processes devices metrics, either added by the blueprint definition
         or the addition using Device.add_metric(). See help(Device.add_metric) for
-        more information about the definition of the metrics to be added
+        more information about the definition of the metrics to be added.
 
         Parameters
         ----------
@@ -466,6 +570,7 @@ class Device(object):
         else: metrics = dict([(key, self.metrics[key]) for key in lmetrics])
 
         for metric in metrics:
+            std_out(f'---')
             std_out(f'Processing {metric}')
 
             if only_new and metric in self.readings:
@@ -481,84 +586,150 @@ class Device(object):
             try:
                 funct = LazyCallable(lazy_name)
             except ModuleNotFoundError:
-                print_exc()
+                #print_exc()
                 process_ok &= False
                 std_out('Problem adding lazy callable to metrics list', 'ERROR')
                 pass
-                return False
 
             args, kwargs = list(), dict()
             if 'args' in metrics[metric]: args = metrics[metric]['args']
             if 'kwargs' in metrics[metric]: kwargs = metrics[metric]['kwargs']
 
             try:
-                self.readings[metric] = funct(self.readings, *args, **kwargs)
+                result = funct(self.readings, *args, **kwargs)
             except KeyError:
                 # print_exc()
                 std_out('Metric args not in dataframe', 'ERROR')
-                process_ok=False
+                process_ok = False
                 pass
-
-            if metric in self.readings: process_ok &= True
+            else:
+                if result is not None:
+                    self.readings[metric] = result
+                    process_ok &= True
+                # If the metric is None, might be for many reasons and shouldn't collapse
+                # the process_ok
 
         if process_ok:
-            # Latest postprocessing to latest readings
             if self.source == 'api':
-                if self.api_device.get_device_postprocessing() is not None:
-                    std_out('Updating postprocessing')
-                    # Add latest postprocessing rounded up with frequency so that we don't end up in
-                    # and endless loop processing only the latest data line (minute vs. second precission of the readings)
-                    latest_postprocessing = localise_date(self.readings.index[-1]+to_timedelta(self.options['frequency']), 'UTC').strftime('%Y-%m-%dT%H:%M:%S')
-                    self.api_device.postprocessing['latest_postprocessing'] = latest_postprocessing
-
-                    std_out(f"{self.api_device.postprocessing}")
+                self.update_latest_postprocessing()
             std_out(f"Device {self.id} processed", "SUCCESS")
 
         self.processed = process_ok
 
-        return process_ok
+        return self.processed
 
-    def forward(self, chunk_size = 500, dry_run = False):
+    def update_latest_postprocessing(self):
+        # Sets latest postprocessing to latest reading
+
+        if self.source == 'api':
+                if self.api_device.get_device_postprocessing() is not None:
+                    std_out('Updating postprocessing')
+                    # Add latest postprocessing rounded up with
+                    # frequency so that we don't end up in
+                    # and endless loop processing only the latest data line
+                    # (minute vs. second precission of the readings)
+                    self.latest_postprocessing = localise_date(self.readings.index[-1]+\
+                        to_timedelta(self.options['frequency']), 'UTC').strftime('%Y-%m-%dT%H:%M:%S')
+                    self.api_device.postprocessing['latest_postprocessing'] = self.latest_postprocessing
+                    std_out(f"Updated latest_postprocessing to: {self.api_device.postprocessing['latest_postprocessing']}")
+
+                    return True
+
+        return False
+
+    def forward(self, chunk_size = 500, dry_run = False, max_retries = 2):
         '''
-            Forwards data to another api
-                Parameters
-                ----------
-                chunk_size: int
-                    500
-                    Chunk size to be sent to device.post_data_to_device in question
-                dry_run: boolean
-                    False
-                    Post the payload to the API or just return it
-            Returns
-            ----------
-                boolean
-                True if posted ok, False otherwise
+        Forwards data to another api.
+        Parameters
+        ----------
+        chunk_size: int
+            500
+            Chunk size to be sent to device.post_data_to_device in question
+        dry_run: boolean
+            False
+            Post the payload to the API or just return it
+        max_retries: int
+            2
+            Maximum number of retries per chunk
+        Returns
+        ----------
+            boolean
+            True if posted ok, False otherwise
         '''
 
-        if self.forwarding_params is None:
-            std_out('Empty forwarding information', 'ERROR')
-            return False
-
-        rd = dict()
-        df = self.readings.copy().dropna(axis = 0, how='all')
-
-        df.rename(columns=rd, inplace=True)
-
-        if df.empty:
-            std_out('Empty dataframe, ignoring', 'WARNING')
-            return False
-
         # Import requested handler
         hmod = __import__('scdata.io.device_api', fromlist = ['io.device_api'])
         Hclass = getattr(hmod, config.connectors[self.forwarding_request]['handler'])
 
-        # Create object
-        device = Hclass(did = self.forwarding_params)
-        post_ok = device.post_data_to_device(df, chunk_size = chunk_size, dry_run = dry_run)
-        if post_ok: std_out(f'Posted data for {self.id}', 'SUCCESS')
-        else: std_out(f'Error posting data for {self.id}', 'ERROR')
+        # Create new device in target API if it hasn't been created yet
+        if self.forwarding_params is None:
+            std_out('Empty forwarding information, attemping creating a new device', 'WARNING')
+            # We assume the device has never been posted
+            # Construct new device kwargs dictionary
+            kwargs = dict()
+            for item in config.connectors[self.forwarding_request]['kwargs']:
+                val = config.connectors[self.forwarding_request]['kwargs'][item]
+                if val == 'options':
+                    kitem = self.options[item]
+                elif val == 'config':
+                    # Items in config should be underscored
+                    kitem = config.__getattr__(f'_{item}')
+                elif isinstance(val, Iterable):
+                    if 'same' in val:
+                        if 'as_device' in val:
+                            if item == 'sensors':
+                                kitem = self.merge_sensor_metrics(ignore_empty = True)
+                            elif item == 'description':
+                                kitem = self.blueprint.replace('_', ' ')
+                        elif 'as_api' in val:
+                            if item == 'sensors':
+                                kitem = self.api_device.get_device_sensors()
+                            elif item == 'description':
+                                kitem = self.api_device.get_device_description()
+                else:
+                    kitem = val
+                kwargs[item] = kitem
 
-        return post_ok
+            response = Hclass.new_device(name = config.connectors[self.forwarding_request]['name_prepend']\
+                                                + str(self.id),
+                                         location = self.location,
+                                         dry_run = dry_run,
+                                         **kwargs)
+            if response:
+                if 'message' in response:
+                    if response['message'] == 'Created':
+                        if 'sensorid' in response:
+                            self.forwarding_params = response['sensorid']
+                            self.api_device.postprocessing['forwarding_params'] = self.forwarding_params
+                            std_out(f'New sensor ID in {self.forwarding_request}\
+                             is {self.forwarding_params}. Updating')
+
+        if self.forwarding_params is not None:
+            df = self.readings.copy()
+            df = df[df.columns.intersection(list(self.merge_sensor_metrics(ignore_empty=True).keys()))]
+            df = clean(df, 'drop', how = 'all')
+
+            if df.empty:
+                std_out('Empty dataframe, ignoring', 'WARNING')
+                return False
+
+            # Create object
+            ndev = Hclass(did = self.forwarding_params)
+            post_ok = ndev.post_data_to_device(df, chunk_size = chunk_size,
+                dry_run = dry_run, max_retries = 2)
+
+            if post_ok:
+                # TODO Check if we like this
+                if self.source == 'api':
+                    self.update_latest_postprocessing()
+                std_out(f'Posted data for {self.id}', 'SUCCESS')
+            else:
+                std_out(f'Error posting data for {self.id}', 'ERROR')
+            return post_ok
+
+        else:
+            std_out('Empty forwarding information', 'ERROR')
+            return False
 
     def add_metric(self, metric = dict()):
         '''
@@ -643,14 +814,22 @@ class Device(object):
             std_out('Not supported format' ,'ERROR')
             return False
 
-    def post_sensors(self, dry_run = False):
+    def post_sensors(self, clean_na = 'drop', chunk_size = 500, dry_run = False, max_retries = 2):
         '''
         Posts devices sensors. Only available for parent of ScApiDevice
             Parameters
             ----------
+            clean_na: string, optional
+                'drop'
+                'drop', 'fill'
+            chunk_size: integer
+                chunk size to split resulting pandas DataFrame for posting readings
             dry_run: boolean
                 False
                 Post the payload to the API or just return it
+            max_retries: int
+                2
+                Maximum number of retries per chunk
         Returns
         ----------
             boolean
@@ -664,7 +843,12 @@ class Device(object):
 
         rd = dict()
         df = self.readings.copy().dropna(axis = 0, how='all')
-        for col in self.readings: rd[col] = self.sensors[col]['id']
+        for col in self.readings:
+            if col not in self.sensors:
+                std_out(f'Column ({col}) not in recognised IDs. Ignoring', 'WARNING')
+                df.drop(col, axis=1, inplace=True)
+                continue
+            rd[col] = self.sensors[col]['id']
 
         df.rename(columns=rd, inplace=True)
 
@@ -672,7 +856,9 @@ class Device(object):
             std_out('Empty dataframe, ignoring', 'WARNING')
             return False
 
-        post_ok = self.api_device.post_data_to_device(df, dry_run = dry_run)
+        std_out(f'Trying to post {list(df.columns)}')
+        post_ok = self.api_device.post_data_to_device(df, clean_na = clean_na,
+            chunk_size = chunk_size, dry_run = dry_run, max_retries = max_retries)
         if post_ok: std_out(f'Posted data for {self.id}', 'SUCCESS')
         else: std_out(f'Error posting data for {self.id}', 'ERROR')
 
@@ -697,7 +883,7 @@ class Device(object):
         if post_ok: std_out(f"Postprocessing posted for device {self.id}", "SUCCESS")
         return post_ok
 
-    def post_metrics(self, with_postprocessing = False, dry_run = False):
+    def post_metrics(self, with_postprocessing = False, chunk_size = 500, dry_run = False, max_retries = 2):
         '''
         Posts devices metrics. Only available for parent of ScApiDevice
         Parameters
@@ -705,9 +891,14 @@ class Device(object):
             with_postprocessing: boolean
                 False
                 Post the postprocessing_attributes too
+            chunk_size: integer
+                chunk size to split resulting pandas DataFrame for posting readings
             dry_run: boolean
                 False
                 Post the payload to the API or just return it
+            max_retries: int
+                2
+                Maximum number of retries per chunk
         Returns
         ----------
             boolean
@@ -726,8 +917,8 @@ class Device(object):
         # Get metrics to post, only the ones that have True in 'post' field and a valid ID
         # Replace their name with the ID to post
         for metric in self.metrics:
-            if self.metrics[metric]['post'] == True:
-                std_out(f"Adding {metric} for device {self.id}")
+            if self.metrics[metric]['post'] == True and metric in self.readings.columns:
+                std_out(f"Adding {metric} for device {self.id} (ID: {self.metrics[metric]['id']})")
                 rd[metric] = self.metrics[metric]['id']
 
         # Keep only metrics in df
@@ -741,7 +932,9 @@ class Device(object):
             std_out('Empty dataframe, ignoring', 'WARNING')
             return False
 
-        post_ok = self.api_device.post_data_to_device(df, dry_run = dry_run)
+        std_out(f'Trying to post {list(df.columns)}')
+        post_ok = self.api_device.post_data_to_device(df, chunk_size = chunk_size,
+            dry_run = dry_run, max_retries = max_retries)
         if post_ok: std_out(f'Posted metrics for {self.id}', 'SUCCESS')
         else: std_out(f'Error posting metrics for {self.id}', 'ERROR')
 
