@@ -1,5 +1,6 @@
 from scdata.utils import std_out, get_units_convf, find_dates, localise_date
 from scdata._config import config
+from scdata.device.process.params import *
 from scdata.device.process import baseline_calc, clean_ts
 from scipy.stats.stats import linregress
 import matplotlib.pyplot as plt
@@ -17,7 +18,7 @@ def alphasense_803_04(dataframe, **kwargs):
         to_date: string, datetime object
             Date until which this calibration id is valid to. None if current
         alphasense_id: string
-            Alphasense sensor ID (must be in calibrations.yaml)
+            Alphasense sensor ID (must be in calibrations.json)
         we: string
             Name of working electrode found in dataframe (V)
         ae: string
@@ -27,8 +28,8 @@ def alphasense_803_04(dataframe, **kwargs):
         use_alternative: boolean
             Default false
             Use alternative algorithm as shown in the AAN
-        location: string
-            Valid location for date localisation
+        timezone: string
+            Valid timezone for date localisation
     Returns
     -------
         calculation of pollutant in ppb
@@ -38,19 +39,19 @@ def alphasense_803_04(dataframe, **kwargs):
         if isnull(x['t']): return None
 
         # Below min temperature, we saturate
-        if x['t'] < config._as_t_comp[0]: return comp_lut[0]
+        if x['t'] < as_t_comp[0]: return comp_lut[0]
 
         # Over max temperature, we saturate
-        if x['t'] > config._as_t_comp[-1]: return comp_lut[-1]
+        if x['t'] > as_t_comp[-1]: return comp_lut[-1]
 
         # Otherwise, we calculate
-        idx_2 = next(axis[0] for axis in enumerate(config._as_t_comp) if axis[1] > x['t'])
+        idx_2 = next(axis[0] for axis in enumerate(as_t_comp) if axis[1] > x['t'])
         idx_1 = idx_2 - 1
 
         delta_y = comp_lut[idx_2] - comp_lut[idx_1]
-        delta_x = config._as_t_comp[idx_2] - config._as_t_comp[idx_1]
+        delta_x = as_t_comp[idx_2] - as_t_comp[idx_1]
 
-        return comp_lut[idx_1] + (x['t'] - config._as_t_comp[idx_1]) * delta_y / delta_x
+        return comp_lut[idx_1] + (x['t'] - as_t_comp[idx_1]) * delta_y / delta_x
 
     # Check inputs
     flag_error = False
@@ -75,17 +76,17 @@ def alphasense_803_04(dataframe, **kwargs):
     # Process input dates
     if 'from_date' not in kwargs: from_date = None
     else:
-        if 'location' not in kwargs:
-            std_out('Cannot localise date without location')
+        if 'timezone' not in kwargs:
+            std_out('Cannot localise date without timezone')
             return None
-        from_date = localise_date(kwargs['from_date'], kwargs['location'])
+        from_date = localise_date(kwargs['from_date'], kwargs['timezone'])
 
     if 'to_date' not in kwargs: to_date = None
     else:
-        if 'location' not in kwargs:
-            std_out('Cannot localise date without location')
+        if 'timezone' not in kwargs:
+            std_out('Cannot localise date without timezone')
             return None
-        to_date = localise_date(kwargs['to_date'], kwargs['location'])
+        to_date = localise_date(kwargs['to_date'], kwargs['timezone'])
 
     # Make copy
     df = dataframe.copy()
@@ -102,9 +103,9 @@ def alphasense_803_04(dataframe, **kwargs):
     else: algorithm_idx = 0
 
     # Get algorithm name
-    algorithm = list(config._as_sensor_algs[as_type].keys())[algorithm_idx]
-    comp_type = config._as_sensor_algs[as_type][algorithm][0]
-    comp_lut = config._as_sensor_algs[as_type][algorithm][1]
+    algorithm = list(as_sensor_algs[as_type].keys())[algorithm_idx]
+    comp_type = as_sensor_algs[as_type][algorithm][0]
+    comp_lut = as_sensor_algs[as_type][algorithm][1]
 
     # Retrieve calibration data - verify its all float
     cal_data = config.calibrations[kwargs['alphasense_id']]
@@ -116,11 +117,11 @@ def alphasense_803_04(dataframe, **kwargs):
             std_out(f'Error on {item}: \'{cal_data[item]}\'', 'ERROR')
             return
 
-    # Remove spurios voltages (0V < electrode < 5V)
+    # Remove spurious voltages (0V < electrode < 5V)
     for electrode in ['we', 'ae']:
         subkwargs = {'name': kwargs[electrode], 
-                     'limits': (0, 5), # In V
-                     'window_size': None
+                        'limits': (0, 5), # In V
+                        'window_size': None
                     }
 
         df[f'{electrode}_clean'] = clean_ts(df, **subkwargs)
@@ -143,6 +144,7 @@ def alphasense_803_04(dataframe, **kwargs):
         df['we_c'] = df['we_t'] - (cal_data['we_sensor_zero_mv'] - cal_data['ae_sensor_zero_mv']) / 1000.0 - df['kp_t'] * df['ae_t']
     elif algorithm == 4:
         df['we_c'] = df['we_t'] - cal_data['we_sensor_zero_mv'] / 1000.0 - df['kpp_t']
+    # TODO - Check if df['we_c'] needs to always be positive and avoid spurious data
 
     # Verify if it has NO2 cross-sensitivity (in V)
     if cal_data['we_cross_sensitivity_no2_mv_ppb'] != float (0):
@@ -152,8 +154,138 @@ def alphasense_803_04(dataframe, **kwargs):
     # Calculate sensor concentration
     df['conc'] = df['we_c'] / (cal_data['we_sensitivity_mv_ppb'] / 1000.0) # in ppb
 
+    if avoid_negative_conc:
+        df['conc'].clip(lower = 0, inplace = True)
+
     return df['conc']
 
+def ec_sensor_temp(dataframe, **kwargs):
+    """
+    Outputs electrochemical temperature
+    Parameters
+    ----------
+        priority: string
+            Name of channel to be used as prioritary
+    Returns
+    -------
+        Temperature series
+    """
+    if 'priority' in kwargs:
+        if kwargs['priority'] in dataframe.columns: return dataframe[kwargs['priority']]
+    if 'ASPT1000' in dataframe.columns: return dataframe['ASPT1000']
+    if 'PM_DALLAS_TEMP' in dataframe.columns: return dataframe['PM_DALLAS_TEMP']
+    std_out('Problem with input data', 'ERROR')
+    return None
+
+def alphasense_pt1000(dataframe, **kwargs):
+    """
+    Calculates temperature in degC of a PT1000, given positive and negative voltage
+    levels (in V), considering negative PT1000 value is grounded
+    Parameters
+    ----------
+        from_date: string, datetime object
+            Date from which this calibration id is valid from
+        to_date: string, datetime object
+            Date until which this calibration id is valid to. None if current
+        pt1000plus: string
+            Name of PT1000+ found in dataframe (V)
+        pt1000minus: string
+            Name of PT1000- found in dataframe (V)
+        timezone: string
+            Valid timezone for date localisation
+        afe_id: string
+            Alphasense AFE ID (must be in calibrations.json)
+    Returns
+    -------
+        Calculation of temperature in degC
+    """
+    # Check inputs
+    flag_error = False
+    if 'pt1000plus' not in kwargs: flag_error = True
+    if 'pt1000minus' not in kwargs: flag_error = True
+
+    if flag_error:
+        std_out('Problem with input data', 'ERROR')
+        return None
+
+    if kwargs['afe_id'] is None:
+        std_out(f"Empty ID. Ignoring", 'WARNING')
+        return None
+
+    # Get Sensor data
+    if kwargs['afe_id'] not in config.calibrations:
+        std_out(f"AFE {kwargs['afe_id']} not in calibration data", 'ERROR')
+        return None
+
+    # Process input dates
+    if 'from_date' not in kwargs: from_date = None
+    else:
+        if 'timezone' not in kwargs:
+            std_out('Cannot localise date without timezone', 'ERROR')
+            return None
+        from_date = localise_date(kwargs['from_date'], kwargs['timezone'])
+
+    if 'to_date' not in kwargs: to_date = None
+    else:
+        if 'timezone' not in kwargs:
+            std_out('Cannot localise date without timezone', 'ERROR')
+            return None
+        to_date = localise_date(kwargs['to_date'], kwargs['timezone'])
+
+    # Retrieve calibration data - verify its all float
+    cal_data = config.calibrations[kwargs['afe_id']]
+    for item in cal_data:
+        try:
+            cal_data[item] = float (cal_data[item])
+        except:
+            std_out(f"Alphasense calibration data for {kwargs['afe_id']} is not correct", 'ERROR')
+            std_out(f'Error on {item}: \'{cal_data[item]}\'', 'ERROR')
+            return
+
+    # Make copy
+    df = dataframe.copy()
+    # Trim data
+    if from_date is not None: df = df[df.index > from_date]
+    if to_date is not None: df = df[df.index < to_date]
+
+    # Calculate temperature
+    df['v20'] = cal_data['v20'] - (cal_data['t20'] - 20.0) / 1000.0
+    df['temp'] = (df[kwargs['pt1000plus']] - df['v20']) * 1000.0 + 20.0 # in degC
+
+    return df['temp']
+
+def channel_names(dataframe, **kwargs):
+    """
+    Gets a named channel related to a generic ADC channel.
+    For instance, an ADC with I2C address 48 has three
+    channels: 0, 1, 2 and 3. 48_0 could corresponde to the working electrode
+    of a CO sensor, so the channel ADC_48_0 would be copied over to a new
+    channel named CO_WE. for the ADC_48_1, the CO auxiliary electrode
+    it would be copied to CO_AE. The naming convention is specified in the
+    hardware folder
+    Parameters
+    ----------
+        channel: string
+            Channel name, i.e. ADC_48_0
+    Returns
+    -------
+        Channel named accordingly
+    """
+    # Check inputs
+    flag_error = False
+    if 'channel' not in kwargs: flag_error = True
+    if kwargs['channel'] not in dataframe:
+        std_out(f"Channel {kwargs['channel']} not in dataframe. Ignoring", 'WARNING')
+        return None
+
+    if flag_error:
+        std_out('Problem with input data', 'ERROR')
+        return None
+
+    # Make copy
+    df = dataframe.copy()
+    return df[kwargs['channel']]
+
 def basic_4electrode_alg(dataframe, **kwargs):
     """
     Calculates pollutant concentration based on 4 electrode sensor readings (mV)
@@ -168,7 +300,7 @@ def basic_4electrode_alg(dataframe, **kwargs):
             Sensor ID
         pollutant: string
             Pollutant name. Must be included in the corresponding LUTs for unit convertion and additional parameters:
-            MOLECULAR_WEIGHTS, config._background_conc, CHANNEL_LUT
+            MOLECULAR_WEIGHTS, background_conc, CHANNEL_LUT
         hardware: alphadelta or isb
     Returns
     -------
@@ -200,7 +332,7 @@ def basic_4electrode_alg(dataframe, **kwargs):
         return None
 
     # This is always in ppm since the calibration data is in signal/ppm
-    if kwargs['hardware'] == 'alphadelta': current_factor = config._alphadelta_pcb
+    if kwargs['hardware'] == 'alphadelta': current_factor = alphadelta_pcb
     elif kwargs['hardware'] == 'isb': current_factor = 1 #TODO make it so we talk in mV
     else: 
         std_out(f"Measurement hardware {kwargs['hardware']} not supported", 'ERROR')
@@ -211,7 +343,7 @@ def basic_4electrode_alg(dataframe, **kwargs):
     # Convert units
     result *= get_units_convf(kwargs['pollutant'], from_units = 'ppm')
     # Add Background concentration
-    result += config._background_conc[kwargs['pollutant']]
+    result += background_conc[kwargs['pollutant']]
 
     return result
 
@@ -232,7 +364,7 @@ def baseline_4electrode_alg(dataframe, **kwargs):
             Sensor ID
         pollutant: string
             Pollutant name. Must be included in the corresponding LUTs for unit convertion and additional parameters:
-            MOLECULAR_WEIGHTS, config._background_conc, CHANNEL_LUT
+            MOLECULAR_WEIGHTS, background_conc, CHANNEL_LUT
         regression_type: 'string'
             'best'
             Use a 'linear' or 'exponential' regression for the calculation of the baseline
@@ -247,7 +379,7 @@ def baseline_4electrode_alg(dataframe, **kwargs):
             '1Min'
             Resample frequency for the target dataframe         
         pcb_factor: int
-            config._alphadelta_pcb (6.36)
+            alphadelta_pcb (6.36)
             Factor converting mV to nA due to the board configuration
 
     Returns
@@ -281,13 +413,13 @@ def baseline_4electrode_alg(dataframe, **kwargs):
     else: resample = '1Min'    
 
     if 'pcb_factor' in kwargs: pcb_factor = kwargs['pcb_factor']
-    else: pcb_factor = config._alphadelta_pcb
+    else: pcb_factor = alphadelta_pcb
     
     if 'baseline_type' in kwargs: baseline_type = kwargs['baseline_type']
     else: baseline_type = 'deltas'
 
     if 'deltas' in kwargs: deltas = kwargs['deltas']
-    else: deltas = config._baseline_deltas
+    else: deltas = baseline_deltas
     
     if flag_error: 
         std_out('Problem with input data', 'ERROR')
@@ -319,7 +451,7 @@ def baseline_4electrode_alg(dataframe, **kwargs):
         # Convert units
         result *= get_units_convf(kwargs['pollutant'], from_units = 'ppm')
         # Add Background concentration
-        result += config._background_conc[kwargs['pollutant']]
+        result += background_conc[kwargs['pollutant']]
     
     else:
         # Calculate non convolved part
@@ -345,7 +477,7 @@ def deconvolution(dataframe, **kwargs):
             Sensor ID
         pollutant: string
             Pollutant name. Must be included in the corresponding LUTs for unit convertion and additional parameters:
-            MOLECULAR_WEIGHTS, config._background_conc, CHANNEL_LUT
+            MOLECULAR_WEIGHTS, background_conc, CHANNEL_LUT
     Returns
     -------
         calculation of pollutant based on: 6.36 * sensitivity(working - zero_working)/(auxiliary - zero_auxiliary)
@@ -377,9 +509,9 @@ def deconvolution(dataframe, **kwargs):
     factor_unit_1 = get_units_convf(kwargs['pollutant'], from_units = 'ppm')
     factor_unit_2 = get_units_convf(kwargs['base'], from_units = 'ppm')
 
-    result = factor_unit_1*(config._alphadelta_pcb*dataframe[kwargs['source']] - dataframe[kwargs['base']]/factor_unit_2*abs(sensitivity_2))/abs(sensitivity_1)
+    result = factor_unit_1*(alphadelta_pcb*dataframe[kwargs['source']] - dataframe[kwargs['base']]/factor_unit_2*abs(sensitivity_2))/abs(sensitivity_1)
     
     # Add Background concentration
-    result += config._background_conc[kwargs['pollutant']]
+    result += background_conc[kwargs['pollutant']]
     
-    return result
\ No newline at end of file
+    return result
