@@ -2,7 +2,7 @@
 
 from os import makedirs
 from os.path import join, exists
-from shutil import copyfile, rmtree
+from shutil import copyfile, rmtree, make_archive
 from traceback import print_exc
 from datetime import datetime
 import yaml
@@ -84,7 +84,8 @@ class Test(object):
             'clean_na',
             'frequency',
             'min_date',
-            'max_date'
+            'max_date',
+            'resample'
         ]
 
         for option in test_options:
@@ -214,14 +215,40 @@ class Test(object):
         if device.id not in self.devices.keys(): self.devices[device.id] = device
         else: std_out(f'Device {device.id} is duplicated', 'WARNING')
 
-    def add_content(self, title = None, figure = None, text = None, iframe = None):
+    def add_content(self, title, figure = None, text = None, iframe = None, show_title = True, force = False):
         '''
-            Adds content for the rendered flask template of the test
+            Adds content for the rendered flask template of the test. Content is a dict()
+            which contains a key per title (replacing ' ' with '_') and the content in it.
+
+            Parameters
+            ----------
+            title
+                None
+                Content title. Needs to not be None
+            figure
+                None
+                matplotlib or similar figure that can be converted to base64
+            text
+                None
+                Text to be converted to <p> html tag with additional inner html (jinja2 safe rendered)
+            iframe
+                None
+                HTML iframe contanining anything
+            show_title
+                True
+                show title in HTML <h3> tag
+            force
+                If already added content with this title
+
+            Returns
+            ----------
+            True if content added, false otherwise
+
         '''
 
         title_cor = sub('\W|^(?=\d)','_', title)
 
-        if title_cor not in self.content:
+        if title_cor not in self.content or force:
             self.content[title_cor] = dict()
 
             if title is not None:
@@ -233,10 +260,13 @@ class Test(object):
             if iframe is not None:
                 self.content[title_cor]['iframe'] = iframe
 
+            self.content[title_cor]['show_title'] = show_title
+
+            std_out('Item added', 'SUCCESS')
             return True
 
         else:
-
+            std_out('Item not added as its already in content', 'ERROR')
             return False
 
     def process(self, only_new = False):
@@ -315,10 +345,17 @@ class Test(object):
                         dst_path = join(self.path, device.processed_data_file)
 
                         # Load csv file, only localising and removing
-                        df = read_csv_file(src_path, device.location, device.frequency, clean_na = None,
-                                           index_name = device.sources[device.source]['index'],
-                                           skiprows = device.sources[device.source]['header_skip'])
-                        df.to_csv(dst_path, sep=",")
+                        df = read_csv_file(file_path = src_path,
+                                            timezone = device.timezone,
+                                            frequency = device.frequency,
+                                            clean_na = None,
+                                            index_name = device.sources[device.source]['index'],
+                                            skiprows = device.sources[device.source]['header_skip'],
+                                            sep = device.sources[device.source]['sep'],
+                                            tzaware = device.sources[device.source]['tz-aware']
+                                            )
+                        df.index.rename(config._csv_defaults['index_name'], inplace=True)
+                        df.to_csv(dst_path, sep=config._csv_defaults['sep'])
 
             std_out('Files preprocessed')
         std_out(f'Test {self.full_name} path: {self.path}')
@@ -351,3 +388,51 @@ class Test(object):
             yaml.dump(self.descriptor, yaml_file)
 
         std_out ('Descriptor file updated')
+
+    def compress(self, cformat = 'zip', selection = 'full'):
+        '''
+            Compress the test folder (or selected folder) into a defined
+            format in the test.path directory
+
+            Parameters
+            ----------
+            cformat
+                'zip'
+                String. Valid shutil.make_archive input: 'zip', 'tar',
+                'gztar', 'bztar', 'xztar'
+            selection
+                'full'
+                String. Selection of folders to compress. Either 'full',
+                'cached' or 'raw'. If 'full', compresses the whole test,
+                including test_description.yaml
+
+            Returns
+            ----------
+            True if all good, False otherwise
+
+        '''
+
+
+        if cformat not in ['zip', 'tar', 'gztar', 'bztar', 'xztar']:
+            std_out('Invalid format', 'ERROR')
+            return False
+
+        if selection not in ['full', 'cached', 'raw']:
+            std_out('Invalid selection (valid options: full, cached, raw', 'ERROR')
+            return False
+
+        if selection == 'full':
+            _root_dir = self.path
+        elif selection == 'cached':
+            _root_dir = join(self.path, 'cached')
+        elif selection == 'raw':
+            _root_dir = join(self.path, 'raw')
+
+        fname_t = join(self.path.strip(f'{self.full_name}')[:-1], self.full_name + f'_{selection}')
+        print (fname_t)
+        make_archive(fname_t, cformat, root_dir=_root_dir)
+
+        fname = fname_t + '.' + cformat
+        if not exists(fname): return False
+
+        return fname
