@@ -38,6 +38,10 @@ def load(self, options = dict()):
                 max_date: String or datetime
                 Default: None
                 Maximum date to load data to
+
+                resample: Boolean
+                Default: True
+                Resample timestamp to the "frequency" resolution
         Returns
         ----------
             None
@@ -60,7 +64,7 @@ def load(self, options = dict()):
     # Set options
     self.__set_options__(options)
 
-    std_out (f'Using options: {self.options}')
+    std_out (f'Using options for test: {self.options}')
 
     for key in self.devices.keys():
 
@@ -68,30 +72,33 @@ def load(self, options = dict()):
         std_out('---------------------------')
         std_out(f'Loading device {device.id}')
 
-        min_date_device = localise_date(device.min_date, device.location)
-        max_date_device = localise_date(device.max_date, device.location)
+        min_date_device = localise_date(device.min_date, device.timezone)
+        max_date_device = localise_date(device.max_date, device.timezone)
 
         # If device comes from API, pre-check dates
         if device.source == 'api':
 
-            if device.location is None:
-                device.location = device.api_device.get_device_timezone()
+            if device.timezone is None:
+                device.timezone = device.api_device.get_device_timezone()
+                min_date_device = localise_date(device.min_date, device.timezone)
+                max_date_device = localise_date(device.max_date, device.timezone)
 
             # Get last reading from API
             if 'get_device_last_reading' in dir(device.api_device):
                 last_reading_api = localise_date(device.api_device.get_device_last_reading(),
-                    device.location)
+                    device.timezone)
 
                 if self.options['load_cached_api']:
                     std_out(f'Checking if we can load cached data')
                     if not device.load(options = self.options,
-                        path = join(self.path, 'cached'), convert_units = False):
+                                       path = join(self.path, 'cached'),
+                                       convert_units = False):
 
                         std_out(f'No valid cached data. Requesting device {device.id} to API', 'WARNING')
                         min_date_to_load = localise_date(device.options['min_date'],
-                                                         device.location)
+                                                         device.timezone)
                         max_date_to_load = localise_date(device.options['max_date'],
-                                                         device.location)
+                                                         device.timezone)
                         load_API = True
 
                     else:
@@ -100,7 +107,7 @@ def load(self, options = dict()):
                         std_out(f'Checking if new data is to be loaded')
 
                         # Get last reading from cached
-                        last_reading_cached = localise_date(device.readings.index[-1], device.location)
+                        last_reading_cached = localise_date(device.readings.index[-1], device.timezone)
                         std_out(f'Last cached date {last_reading_cached}')
                         std_out(f'Last reading in API {last_reading_api}')
 
@@ -180,11 +187,14 @@ def load(self, options = dict()):
                 if 'frequency' in self.options:
                     device_options['frequency'] = self.options['frequency']
 
+                if 'resample' in self.options:
+                    device_options['resample'] = self.options['resample']
+
                 device.load(options = device_options)
 
         elif device.source == 'csv':
 
-            device.load(options = self.options, path = self.path)
+            device.load(options = self.options, path = self.path, follow_defaults = True)
 
         if self.options['store_cached_api'] and device.loaded and device.source == 'api' and load_API:
 
@@ -195,11 +205,13 @@ def load(self, options = dict()):
                 std_out('Creating path for exporting cached data')
                 makedirs(cached_file_path)
 
-            if export_csv_file(cached_file_path, device.id, device.readings, forced_overwrite = True): std_out('Devices cached')
+            if export_csv_file(cached_file_path, device.id, device.readings, forced_overwrite = True):
+                std_out('Devices cached', 'SUCCESS')
 
         if device.loaded: std_out(f'Device {device.id} has been loaded', 'SUCCESS')
         else: std_out(f'Could not load device {device.id}. Skipping', 'WARNING')
 
+    # TODO IMPORTANT!
     self.__update_descriptor__()
     std_out('Test load done', 'SUCCESS')
     self.loaded = True
