@@ -1,5 +1,7 @@
 from requests import get
 from pandas import json_normalize
+from scdata.utils import std_out
+from scdata._config import config
 
 def get_elevation(_lat = None, _long = None):
     '''
@@ -13,12 +15,28 @@ def get_elevation(_lat = None, _long = None):
     
     query = ('https://api.open-elevation.com/api/v1/lookup'
              f'?locations={_lat},{_long}')
+
     # Request with a timeout for slow responses
-    r = get(query, timeout = 20)
+    error = False
+    try:
+        r = get(query, timeout = config._timeout)
+    except:
+        std_out(f'Cannot get altitude from {query}')
+        error = True
+        pass
+
+    if error: return None
 
     # Only get the json response in case of 200 or 201
     if r.status_code == 200 or r.status_code == 201:
         elevation = json_normalize(r.json(), 'results')['elevation'].values[0]
     else:
         elevation = None
-    return elevation
\ No newline at end of file
+
+    try:
+        elevation = int(elevation)
+    except:
+        elevation = None
+        pass
+
+    return elevation
